# -*- coding: utf-8 -*- 
import autopath
import os.path as osp
import os
import datetime

from logparser import CSVStorage, filter_schedule, parselog, filter_denied
from tools import isoformat_to_datetime

DATA = osp.join(autopath.HERE, 'data')

## test des objets de bas niveau

def test_read_data():
    """le parseur, le format des données
    lecture des données dans un fichier de types acces.log"""
    keywords = ['date','login','ip', 'url', 'method', 'chiffre1', 'chiffre2']
    c = CSVStorage(keywords=keywords, filename=osp.join(DATA, 'acces.log')) 
    c.load_data()
    expected = [{'url': 'http://fr.fxfeeds.mozilla.com/fr/firefox/headlines.xml', 'ip': '10.21.11.30', 'chiffre2': '338', 'date': '2007.6.23 9:02:00', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://www.sex.com/', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.23 9:02:38', 'login': '-', 'method': '*DENIED* Site interdit: sex.com', 'chiffre1': 'GET'}, {'url': 'http://sexocean.com/', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.23 9:02:44', 'login': '-', 'method': '*DENIED* Site interdit: sexocean.com', 'chiffre1': 'GET'}, {'url': 'http://sexocean.com/favicon.ico', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.23 9:02:44', 'login': '-', 'method': '*DENIED* Site interdit: sexocean.com', 'chiffre1': 'GET'}, {'url': 'http://sexe.com/', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.23 9:03:01', 'login': '-', 'method': '*DENIED* Site interdit: sexe.com', 'chiffre1': 'GET'}, {'url': 'http://sexe.com/favicon.ico', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.23 9:03:02', 'login': '-', 'method': '*DENIED* Site interdit: sexe.com', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/', 'ip': '10.21.11.30', 'chiffre2': '271', 'date': '2007.6.23 9:03:07', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/pub/', 'ip': '10.21.11.30', 'chiffre2': '67387', 'date': '2007.6.23 9:03:27', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/pub/', 'ip': '10.21.11.30', 'chiffre2': '67386', 'date': '2007.6.23 9:03:30', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/css/style.css', 'ip': '10.21.11.30', 'chiffre2': '12583', 'date': '2007.6.23 9:03:30', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/js/admin.js', 'ip': '10.21.11.30', 'chiffre2': '11638', 'date': '2007.6.23 9:03:30', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/css/contrib/kaiska.css', 'ip': '10.21.11.30', 'chiffre2': '17866', 'date': '2007.6.23 9:03:31', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/css/linuxfr_print.css', 'ip': '10.21.11.30', 'chiffre2': '834', 'date': '2007.6.23 9:03:31', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://sb.google.com/safebrowsing/update?client=navclient-auto-ffox2.0.0.3&mozver=1.8.1.3-2007030919&version=goog-white-domain:1:23,goog-white-url:1:371,goog-black-url:1:11729,goog-black-enchash:1:27574', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.15 12:19:26', 'login': '-', 'method': '', 'chiffre1': 'GET'}]
    assert c.get_data() == expected

def test_read_zipped_data():
    """lecture des données dans un fichier de type acces.log en compressé (zippé)"""
    keywords = ['date','login','ip', 'url', 'method', 'chiffre1', 'chiffre2']
    c = CSVStorage(keywords=keywords, filename=osp.join(DATA, 'acces.log.gz')) 
    c.load_data()
    expected = [{'url': 'http://fr.fxfeeds.mozilla.com/fr/firefox/headlines.xml', 'ip': '10.21.11.30', 'chiffre2': '338', 'date': '2007.6.23 9:02:00', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://www.sex.com/', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.23 9:02:38', 'login': '-', 'method': '*DENIED* Site interdit: sex.com', 'chiffre1': 'GET'}, {'url': 'http://sexocean.com/', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.23 9:02:44', 'login': '-', 'method': '*DENIED* Site interdit: sexocean.com', 'chiffre1': 'GET'}, {'url': 'http://sexocean.com/favicon.ico', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.23 9:02:44', 'login': '-', 'method': '*DENIED* Site interdit: sexocean.com', 'chiffre1': 'GET'}, {'url': 'http://sexe.com/', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.23 9:03:01', 'login': '-', 'method': '*DENIED* Site interdit: sexe.com', 'chiffre1': 'GET'}, {'url': 'http://sexe.com/favicon.ico', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.23 9:03:02', 'login': '-', 'method': '*DENIED* Site interdit: sexe.com', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/', 'ip': '10.21.11.30', 'chiffre2': '271', 'date': '2007.6.23 9:03:07', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/pub/', 'ip': '10.21.11.30', 'chiffre2': '67387', 'date': '2007.6.23 9:03:27', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/pub/', 'ip': '10.21.11.30', 'chiffre2': '67386', 'date': '2007.6.23 9:03:30', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/css/style.css', 'ip': '10.21.11.30', 'chiffre2': '12583', 'date': '2007.6.23 9:03:30', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/js/admin.js', 'ip': '10.21.11.30', 'chiffre2': '11638', 'date': '2007.6.23 9:03:30', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/css/contrib/kaiska.css', 'ip': '10.21.11.30', 'chiffre2': '17866', 'date': '2007.6.23 9:03:31', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/css/linuxfr_print.css', 'ip': '10.21.11.30', 'chiffre2': '834', 'date': '2007.6.23 9:03:31', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://sb.google.com/safebrowsing/update?client=navclient-auto-ffox2.0.0.3&mozver=1.8.1.3-2007030919&version=goog-white-domain:1:23,goog-white-url:1:371,goog-black-url:1:11729,goog-black-enchash:1:27574', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.15 12:19:26', 'login': '-', 'method': '', 'chiffre1': 'GET'}]
    assert c.get_data() == expected

def _motif_in_string(motif, expected):
    assert motif in expected

def test_filter():
    "objet logparser: filtre des données contenant des sites déjà interdits par dansguardian"
    keywords = ['date','login','ip', 'url', 'exception', 'method', 'chiffre1']
    c = CSVStorage(keywords=keywords, filename=osp.join(DATA, 'acces.log')) 
    c.load_data()
    # il y a bien un site interdit dans les résultat
    res = [data['exception'] for data in c.get_filtered()]
    res = [data for data in res if data]
    for result in res:
        _motif_in_string('*DENIED* Site interdit', result)

    # filtrage
    c.filter('exception', '*DENIED*')
    res = [data['exception'] for data in c.get_filtered()]
    for r in res:
        assert '*DENIED* Site interdit' in r

## test des fonctions de haut niveau

def test_filter_schedule():
    "filtre des données contenant des sites déjà interdits"
    keywords = ['date','login','ip', 'url', 'exception', 'method', 'chiffre1']
    c = CSVStorage(keywords=keywords, filename=osp.join(DATA, 'acces.log')) 
    c.load_data()
    begin, end =  isoformat_to_datetime("2007.6.23 9:02:00"),  isoformat_to_datetime("2007.6.23 9:03:55")
    expected = [{'exception': '*DENIED* Site interdit: sex.com', 'url': 'http://www.sex.com/', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 2, 38), 'login': '-', 'method': 'GET', 'chiffre1': '0'}, {'exception': '*DENIED* Site interdit: sexocean.com', 'url': 'http://sexocean.com/', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 2, 44), 'login': '-', 'method': 'GET', 'chiffre1': '0'}, {'exception': '*DENIED* Site interdit: sexocean.com', 'url': 'http://sexocean.com/favicon.ico', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 2, 44), 'login': '-', 'method': 'GET', 'chiffre1': '0'}, {'exception': '*DENIED* Site interdit: sexe.com', 'url': 'http://sexe.com/', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 1), 'login': '-', 'method': 'GET', 'chiffre1': '0'}, {'exception': '*DENIED* Site interdit: sexe.com', 'url': 'http://sexe.com/favicon.ico', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 2), 'login': '-', 'method': 'GET', 'chiffre1': '0'}, {'exception': '', 'url': 'http://linuxfr.org/', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 7), 'login': '-', 'method': 'GET', 'chiffre1': '271'}, {'exception': '', 'url': 'http://linuxfr.org/pub/', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 27), 'login': '-', 'method': 'GET', 'chiffre1': '67387'}, {'exception': '', 'url': 'http://linuxfr.org/pub/', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 30), 'login': '-', 'method': 'GET', 'chiffre1': '67386'}, {'exception': '', 'url': 'http://linuxfr.org/css/style.css', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 30), 'login': '-', 'method': 'GET', 'chiffre1': '12583'}, {'exception': '', 'url': 'http://linuxfr.org/js/admin.js', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 30), 'login': '-', 'method': 'GET', 'chiffre1': '11638'}, {'exception': '', 'url': 'http://linuxfr.org/css/contrib/kaiska.css', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 31), 'login': '-', 'method': 'GET', 'chiffre1': '17866'}, {'exception': '', 'url': 'http://linuxfr.org/css/linuxfr_print.css', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 31), 'login': '-', 'method': 'GET', 'chiffre1': '834'}]

    assert list(c.filter_schedule(begin, end)) == expected

def test_filter_schedule2():
    """filtres les données dans un intervalle de temps considéré"""
    keywords = ['date','login','ip', 'url', 'exception', 'method', 'chiffre1']
    begin, end =  isoformat_to_datetime("2007.6.23 9:02:00"),  isoformat_to_datetime("2007.6.23 9:03:55")
    expected = [{'exception': '*DENIED* Site interdit: sex.com', 'url': 'http://www.sex.com/', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 2, 38), 'login': '-', 'method': 'GET', 'chiffre1': '0'}, {'exception': '*DENIED* Site interdit: sexocean.com', 'url': 'http://sexocean.com/', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 2, 44), 'login': '-', 'method': 'GET', 'chiffre1': '0'}, {'exception': '*DENIED* Site interdit: sexocean.com', 'url': 'http://sexocean.com/favicon.ico', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 2, 44), 'login': '-', 'method': 'GET', 'chiffre1': '0'}, {'exception': '*DENIED* Site interdit: sexe.com', 'url': 'http://sexe.com/', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 1), 'login': '-', 'method': 'GET', 'chiffre1': '0'}, {'exception': '*DENIED* Site interdit: sexe.com', 'url': 'http://sexe.com/favicon.ico', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 2), 'login': '-', 'method': 'GET', 'chiffre1': '0'}, {'exception': '', 'url': 'http://linuxfr.org/', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 7), 'login': '-', 'method': 'GET', 'chiffre1': '271'}, {'exception': '', 'url': 'http://linuxfr.org/pub/', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 27), 'login': '-', 'method': 'GET', 'chiffre1': '67387'}, {'exception': '', 'url': 'http://linuxfr.org/pub/', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 30), 'login': '-', 'method': 'GET', 'chiffre1': '67386'}, {'exception': '', 'url': 'http://linuxfr.org/css/style.css', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 30), 'login': '-', 'method': 'GET', 'chiffre1': '12583'}, {'exception': '', 'url': 'http://linuxfr.org/js/admin.js', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 30), 'login': '-', 'method': 'GET', 'chiffre1': '11638'}, {'exception': '', 'url': 'http://linuxfr.org/css/contrib/kaiska.css', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 31), 'login': '-', 'method': 'GET', 'chiffre1': '17866'}, {'exception': '', 'url': 'http://linuxfr.org/css/linuxfr_print.css', 'ip': '10.21.11.30', 'date': datetime.datetime(2007, 6, 23, 9, 3, 31), 'login': '-', 'method': 'GET', 'chiffre1': '834'}]
    assert filter_schedule(keywords, begin, end, fname=osp.join(DATA, 'acces.log')) == expected
    
# def test_logparser():
#     """test de la methode de parsing des logs"""
#     keywords = ['date','login','ip', 'url', 'exception', 'method', 'chiffre1']
#     expected = [{'url': 'http://fr.fxfeeds.mozilla.com/fr/firefox/headlines.xml', 'ip': '10.21.11.30', 'chiffre2': '338', 'date': '2007.6.23 9:02:00', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://www.sex.com/', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.23 9:02:38', 'login': '-', 'method': '*DENIED* Site interdit: sex.com', 'chiffre1': 'GET'}, {'url': 'http://sexocean.com/', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.23 9:02:44', 'login': '-', 'method': '*DENIED* Site interdit: sexocean.com', 'chiffre1': 'GET'}, {'url': 'http://sexocean.com/favicon.ico', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.23 9:02:44', 'login': '-', 'method': '*DENIED* Site interdit: sexocean.com', 'chiffre1': 'GET'}, {'url': 'http://sexe.com/', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.23 9:03:01', 'login': '-', 'method': '*DENIED* Site interdit: sexe.com', 'chiffre1': 'GET'}, {'url': 'http://sexe.com/favicon.ico', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.23 9:03:02', 'login': '-', 'method': '*DENIED* Site interdit: sexe.com', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/', 'ip': '10.21.11.30', 'chiffre2': '271', 'date': '2007.6.23 9:03:07', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/pub/', 'ip': '10.21.11.30', 'chiffre2': '67387', 'date': '2007.6.23 9:03:27', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/pub/', 'ip': '10.21.11.30', 'chiffre2': '67386', 'date': '2007.6.23 9:03:30', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/css/style.css', 'ip': '10.21.11.30', 'chiffre2': '12583', 'date': '2007.6.23 9:03:30', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/js/admin.js', 'ip': '10.21.11.30', 'chiffre2': '11638', 'date': '2007.6.23 9:03:30', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/css/contrib/kaiska.css', 'ip': '10.21.11.30', 'chiffre2': '17866', 'date': '2007.6.23 9:03:31', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://linuxfr.org/css/linuxfr_print.css', 'ip': '10.21.11.30', 'chiffre2': '834', 'date': '2007.6.23 9:03:31', 'login': '-', 'method': '', 'chiffre1': 'GET'}, {'url': 'http://sb.google.com/safebrowsing/update?client=navclient-auto-ffox2.0.0.3&mozver=1.8.1.3-2007030919&version=goog-white-domain:1:23,goog-white-url:1:371,goog-black-url:1:11729,goog-black-enchash:1:27574', 'ip': '10.21.11.30', 'chiffre2': '0', 'date': '2007.6.15 12:19:26', 'login': '-', 'method': '', 'chiffre1': 'GET'}]
#     result = parselog(osp.join(DATA, 'acces.log'), keywords)
#     assert result == expected

def test_filter_denied():
    pass

def test_parse_denied():
    pass

